@@ -1,13 +1,12 @@
 from coldtype import *
 from functools import partial
 from random import Random
+from pprint import pprint
 
 
-co = Font("assets/ColdtypeObviously-VF.ttf")
-
-mutator = Font("assets/MutatorSans.ttf")
-
-recmono = Font("viewer/assets/RecMono-CasualItalic.ttf")
+co = Font.Cacheable("assets/ColdtypeObviously-VF.ttf")
+mutator = Font.Cacheable("assets/MutatorSans.ttf")
+recmono = Font.Cacheable("assets/RecMono-CasualItalic.ttf")
 
 Style.RegisterShorthandPrefix("≈", "~/Type/fonts/fonts")
 
@@ -15,7 +14,7 @@ Style.RegisterShorthandPrefix("≈", "~/Type/fonts/fonts")
 def add_grid(render, result):
     return DATPenSet([
         DATPenSet(result),
-        DATPen().gridlines(render.rect).s(0, 0.1)
+        DATPen().gridlines(render.rect).s(0, 0.1).sw(1)
     ])
 
 
