@@ -15,6 +15,8 @@ from coldtype.color import normalize_color
 from coldtype.pens.datpen import DATPen, DATPenSet
 from coldtype.geometry import Rect, Point
 
+from typing import Optional, Callable, Union
+
 try:
     from fontgoggles.font import getOpener
     from fontgoggles.font.baseFont import BaseFont
@@ -32,6 +34,7 @@ class FittableMixin():
         print("textContent() not overwritten")
 
     def fit(self, width):
+        """Use various methods (tracking, `wdth` axis, etc. — properties specified in the `Style` object) to fit a piece of text horizontally to a given `width` (warning: not very fast)"""
         if isinstance(width, Rect):
             width = width.w
         current_width = self.width()
@@ -76,106 +79,137 @@ def normalize_font_path(font):
 def empty_writer(*args):
     pass
 
+FontCache = {}
+
 class Font():
     # TODO support glyphs?
-    def __init__(self, path, number=0):
+    def __init__(self, path, number=0, cacheable=False):
         self.path = Path(normalize_font_path(path))
         numFonts, opener, getSortInfo = getOpener(self.path)
         self.font:BaseFont = opener(self.path, number)
         self.font.cocoa = False
+        self.cacheable = cacheable
+        self._loaded = False
+        self.load()
     
     def load(self):
-        asyncio.get_event_loop().run_until_complete(self.font.load(empty_writer))
-        return self
+        if self._loaded:
+            return self
+        else:
+            self._loaded = True
+            asyncio.get_event_loop().run_until_complete(self.font.load(empty_writer))
+            return self
     
-    def Preload(path):
-        return Font(path).load()
+    def Cacheable(path):
+        if path not in FontCache:
+            FontCache[path] = Font(path, cacheable=True).load()
+        return FontCache[path]
 
 class Style():
+    """
+    Class for configuring font properties
+
+    **Keyword arguments**
+
+    * ``font``: can either be a ``coldtype.text.Font`` object, a ``pathlib.Path``, or a plain string path
+    * ``fontSize``: standard point-based font-size, expressed as integer
+    * ``tracking`` (aka ``tu``): set the tracking, by default **in font-source-point-size** aka as if the font-size was always 1000; this means tracking is by default done relatively rather than absolutely (aka the relative tracking will not change when you change the fontSize)
+    * ``trackingMode``: set to 0 to set tracking in a classic fontSize-based (defaults to 1, as described just above)
+    * ``space``: set this to override the width of the standard space character (useful when setting text on a curve and the space is getting collapsed)
+    * ``baselineShift`` (aka ``bs``): if an integer, shifts glyphs by that amount in y axis; if a list, shifts glyphs at corresponding index in list by that amount in y axis
+    * ``xShift`` (aka ``xs``): if an integer, shifts glyphs by that amount in x axis; if a list, shifts glyphs at corresponding index in list by that amount in x axis
+    * ``rotate``: rotate glyphs by degree
+    * ``reverse`` (aka ``r``): reverse the order of the glyphs, so that the left-most glyph is first in when vectorized via ``.pens()``
+    * ``removeOverlaps`` (aka ``ro``): automatically use skia-pathops to remove overlaps from the glyphs (useful when using variable ttf fonts)
+    * ``lang``: set language directly, to access language-specific alternate characters/rules
+
+    **Shorthand kwargs**
+
+    * ``kp`` for ``kern_pairs`` — a dict of glyphName->[left,right] values in font-space
+    * ``tl`` for ``trackingLimit``
+    * ``bs`` for ``baselineShift``
+    * ``ch`` for ``capHeight`` — a number in font-space; not specified, read from font; specified as 'x', capHeight is set to xHeight as read from font
+    """
     def RegisterShorthandPrefix(prefix, expansion):
         global _prefixes
         _prefixes.append([prefix, str(expansion)])
 
     def __init__(self,
-            font=None,
-            fontSize=12,
-            fitHeight=None,
+            font:Union[Font, str]=None,
+            fontSize:int=12,            
             tracking=0,
-            trackingLimit=0,
             trackingMode=1,
             kern_pairs=dict(),
-            overlap_pairs=dict(),
-            overlap_outline=3,
             space=None,
             baselineShift=0,
             xShift=None,
-            mods=None,
-            variations=dict(),
-            variationLimits=dict(),
-            increments=dict(),
-            features=dict(),
-            liga=True,
-            varyFontSize=False,
-            fill=(0, 0.5, 1),
-            stroke=None,
-            strokeWidth=0,
+            rotate=0,
+            reverse=False,
+            removeOverlap=False,
+            lang=None,
+            narrower=None,
+            include_blanks=False,
             palette=0,
             capHeight=None,
             data={},
-            latin=None, # temp
-            lang=None,
-            filter=None,
-            layerer=None,
-            preventHwid=False,
             layer=None,
-            reverse=False,
-            removeOverlap=False,
-            rotate=0,
-            sv=True,
-            narrower=None,
+            liga=True,
+            fill=(0, 0.5, 1),
+            stroke=None,
+            strokeWidth=0,
+            variations=dict(),
+            variationLimits=dict(),
+            trackingLimit=0,
+            scaleVariations=True,
+            mods=None,
+            features=dict(),
+            increments=dict(),
+            varyFontSize=False,
+            preventHwid=False,
+            fitHeight=None,
+            load_font=True, # should Coldtype attempt to load the font?
+            tag=None, # way to differentiate in __eq__
             **kwargs):
-        """
-        kern_pairs (kp) — a dict of glyphName->[left,right] values in font-space
-        tracking (t)
-        trackingLimit (tl)
-        baselineShift (bs)
-        capHeight (ch) — A number in font-space; not specified, read from font; specified as 'x', capHeight is set to xHeight as read from font
-        """
 
         self.input = locals()
         self.input["self"] = None
 
-        if isinstance(font, str):
-            self.font:Font = Font(font)
-            if isinstance(self.font.font, OTFFont):
+        if load_font:
+            if isinstance(font, Path):
+                font = str(font)
+            if isinstance(font, str):
+                self.font:Font = Font(font)
                 self.font.load() #font._syncLoad(None)
             else:
-                print("Editable font formats cannot be loaded synchronously")
-
+                self.font:Font = font
         else:
-            self.font:Font = font
+            self.font = font
 
         self.narrower = narrower
         self.layer = layer
-        self.layerer = layerer
         self.reverse = kwargs.get("r", reverse)
         self.removeOverlap = kwargs.get("ro", removeOverlap)
         self.rotate = rotate
-        self.sv = sv # scale-variations
-        
-        if "OS/2" in self.font.font.ttFont:
-            os2 = self.font.font.ttFont["OS/2"]
-            self.capHeight = os2.sCapHeight if hasattr(os2, "sCapHeight") else 0
-            if self.capHeight == 0:
-                self.capHeight = os2.sTypoAscender if hasattr(os2, "sTypoAscender") else 750
-        elif hasattr(self.font.font, "info"):
-            self.capHeight = self.font.font.info.capHeight
-        elif hasattr(self.font.font, "defaultInfo"):
-            self.capHeight = self.font.font.defaultInfo.capHeight
+        self.include_blanks = include_blanks
+        self.scaleVariations = scaleVariations
+        self.tag = tag
 
-        if capHeight: # override whatever the font says
-            if capHeight != "x":
-                self.capHeight = capHeight
+        try:        
+            if "OS/2" in self.font.font.ttFont:
+                os2 = self.font.font.ttFont["OS/2"]
+                self.capHeight = os2.sCapHeight if hasattr(os2, "sCapHeight") else 0
+                if self.capHeight == 0:
+                    self.capHeight = os2.sTypoAscender if hasattr(os2, "sTypoAscender") else 750
+            elif hasattr(self.font.font, "info"):
+                self.capHeight = self.font.font.info.capHeight
+            elif hasattr(self.font.font, "defaultInfo"):
+                self.capHeight = self.font.font.defaultInfo.capHeight
+
+            if capHeight: # override whatever the font says
+                if capHeight != "x":
+                    self.capHeight = capHeight
+        except AttributeError:
+            pass
 
         if fitHeight:
             self.fontSize = (fitHeight/self.capHeight)*1000
@@ -184,8 +218,6 @@ class Style():
 
         self.tracking = kwargs.get("t", tracking)
         self.kern_pairs = kwargs.get("kp", kern_pairs)
-        self.overlap_pairs = overlap_pairs
-        self.overlap_outline = overlap_outline
         self.trackingMode = trackingMode
         self.trackingLimit = kwargs.get("tl", trackingLimit)
         self.baselineShift = kwargs.get("bs", baselineShift)
@@ -195,9 +227,7 @@ class Style():
         self.mods = mods
         self.palette = palette
         self.lang = lang
-        self.filter = filter
         self.data = data
-        self.latin = latin
         self.preventHwid = preventHwid
 
         if kwargs.get("tu"):
@@ -231,7 +261,7 @@ class Style():
         self.variationLimits = dict()
         self.varyFontSize = varyFontSize
         
-        if self.font.font.ttFont:
+        if load_font and self.font.font.ttFont:
             try:
                 fvar = self.font.font.ttFont['fvar']
             except:
@@ -248,21 +278,34 @@ class Style():
             self.addVariations(unnormalized_variations)
     
     def __eq__(self, other):
+        if not self.tag == other.tag:
+            return False
         if not self.font == other.font:
+            #print("different font")
             return False
         elif not self.fontSize == other.fontSize:
+            #print("different fontSize")
             return False
         
         for key, value in self.variations.items():
             if self.variations[key] != other.variations[key]:
+                #print("different variation")
                 return False
+            
+        if self.rotate != other.rotate:
+            #print("different rotate")
+            return False
         
         if self.fill != other.fill:
+            #print("different fill")
             return False
         
         return True
 
     def mod(self, **kwargs):
+        """Modify this style object to create a new
+        one; kwargs can have all of the same kwargs as
+        the standard `Style` constructor"""
         keyed = dict(**self.input, **self.input["kwargs"])
         del keyed["kwargs"]
         del keyed["self"]
@@ -276,7 +319,7 @@ class Style():
             self.variationLimits[k] = v
     
     def normalizeVariations(self, variations):
-        scale = self.sv
+        scale = self.scaleVariations
         for k, v in variations.items():
             try:
                 axis = self.axes[k]
@@ -400,7 +443,10 @@ def offset(x, y, ox, oy):
 
 
 class StyledString(FittableMixin):
-    def __init__(self, text, style):
+    """
+    Lowest-level vectorized typesetting class
+    """
+    def __init__(self, text:str, style:Style):
         self.text_info = TextInfo(text)
         self.text = text
         self.setStyle(style)
@@ -425,7 +471,7 @@ class StyledString(FittableMixin):
             x += glyph.ax
         self.getGlyphFrames()
     
-    def trackFrames(self):
+    def trackFrames(self, space_width=0):
         t = self.tracking
         x_off = 0
         for idx, g in enumerate(self.glyphs):
@@ -434,7 +480,7 @@ class StyledString(FittableMixin):
             if self.style.mods and g.name in self.style.mods:
                 x_off += self.style.mods[g.name][0]
             if self.style.space and g.name.lower() == "space":
-                x_off += self.style.space
+                x_off += (self.style.space - space_width)
     
     def adjustFramesForPath(self):
         for idx, g in enumerate(self.glyphs):    
@@ -450,6 +496,7 @@ class StyledString(FittableMixin):
             last_gn = None
             for idx, glyph in enumerate(self.glyphs):
                 gn = glyph.name
+
                 for chars, kp in self.style.kern_pairs.items():
                     try:
                         l = kp[0]
@@ -468,8 +515,14 @@ class StyledString(FittableMixin):
                                 glyph.frame.x += kern_shift
                 last_gn = gn
         
+        space_width = 0
+        for glyph in self.glyphs:
+            if glyph.name == "space" and self.style.space and self.style.space > 0:
+                space_width = glyph.frame.w
+                glyph.frame.w = self.style.space
+
         if self.style.trackingMode == 1:
-            self.trackFrames()
+            self.trackFrames(space_width=space_width)
 
         for glyph in self.glyphs:
             glyph.frame = glyph.frame.scale(self.scale())
@@ -645,6 +698,9 @@ class StyledString(FittableMixin):
         return dp
 
     def pens(self, frame=True) -> DATPenSet:
+        """
+        Vectorize text into a ``DATPenSet``, such that each glyph (or ligature) is represented by a single `DATPen` (or a ``DATPenSet`` in the case of a color font, which will then nest a `DATPen` for each layer of that color glyph)
+        """
         self.resetGlyphRun()
         self.style.font.font.addGlyphDrawings(self.glyphs, colorLayers=True)
         
@@ -669,17 +725,20 @@ class StyledString(FittableMixin):
                     dp_atom += dp_layer
                 dp_atom.addFrame(g.frame, typographic=True)
                 dp_atom.glyphName = g.name
-            pens.append(dp_atom)
+            pens.append(dp_atom, allow_blank=self.style.include_blanks)
 
         if self.style.reverse:
             pens.reversePens()
         
-        if self.style.kern_pairs:
-            overlap_pairs = {pair:value[2] for (pair, value) in self.style.kern_pairs.items() if not isinstance(value, int) and len(value) > 2}
-            pens.overlapPairs(overlap_pairs, outline=self.style.overlap_outline)
+        for k, v in self.style.data.items():
+            pens.data[k] = v
+
         return pens
 
     def pen(self, frame=True) -> DATPen:
+        """
+        Vectorize all text into single ``DATPen``
+        """
         return self.pens(frame=frame).pen()
 
 class SegmentedString(FittableMixin):
