@@ -1,6 +1,6 @@
 from coldtype.test import *
 
-tl = Timeline(26, fps=23.976, storyboard=[0])
+tl = Timeline(26, fps=23.976, storyboard=[0], jumps=[5])
 
 def find_workarea(self):
     return [0, 1, 2]
@@ -20,5 +20,4 @@ def render(f):
             Style(recmono, 72, wdth=1))
             .pens()
             .align(f.a.r.take(150, "mny"), th=0)
-            .f(hsl((1-pe)+0.5)))
-    ])
\ No newline at end of file
+            .f(hsl((1-pe)+0.5)))])
\ No newline at end of file
