@@ -1,19 +1,22 @@
 from coldtype import *
 
 co = Font("assets/ColdtypeObviously.designspace")
-
 img = Path("test/renders/test_skia_image_is_image.png")
 
 @renderable(rect=(1000, 1000))
 def uses_image(r):
     return DATPenSet([
-        DATPen().oval(r.inset(10)).f(hsl(0.5, l=0.2)),
+        DATPen().oval(r.inset(10)).f(hsl(0.5, l=0.5)),
         (DATPen()
             .oval(r.inset(20))
             .f(None)
-            .attr(image=dict(src=img, opacity=0.35, rect=Rect(0, 0, 125, 125))))])
+            .attr(image=dict(
+                src=img,
+                opacity=0.5,
+                pattern=True,
+                rect=r.take(100, "mnx").square())))])
 
-#@renderable(rect=(250, 250))
+@renderable(rect=(250, 250))
 def is_image(r):
     return DATPenSet([
         DATPen().oval(r.inset(5)).f(hsl(random())),
@@ -25,21 +28,31 @@ def is_image(r):
             .translate(0, 10)
             .f(1))])
 
-#@renderable(rect=(500, 500))
+@renderable(rect=(500, 500))
 def uses_shadow(r):
     return DATPenSet([
         (DATPen()
             .rect(r.inset(100))
             .rotate(45)
-            .attr(shadow=dict(clip=r.inset(100), alpha=0.75, radius=30))
+            .shadow(30, (0, 0.75), r.inset(100))
+            #.attr(shadow=dict(clip=r.inset(100), alpha=0.75, radius=30))
             .f(hsl(0.75))),
         (DATPen()
             .rect(r.take(20, "mdx"))
             .f(hsl(0.5))
             .rotate(-45)
-            .attr(shadow=dict(alpha=1, radius=30))),
+            .shadow(30, (0, 1))),
         (DATPen()
             .rect(r.take(20, "mdy"))
             .f(hsl(0.1, s=0.6, l=0.6))
-            .attr(shadow=dict(alpha=1, radius=30)))
-    ])
\ No newline at end of file
+            .shadow(30, (0, 1)))
+    ])
+
+@renderable(rect=(500, 500), solo=0)
+def placed_precompose(r):
+    dp = DATPen().oval(r.inset(140)).f(hsl(0.5))
+    dpr = dp.precompose(
+        r.inset(150)#.zero(),
+        #r.inset(150)
+        )
+    return dpr
